/* Rain.rexx -- Precipitation Particle Program */
/*    by Arnie... */
/* Based on Crude particle system program by Allen Hastings */
/* Wed Dec 23 00:21:15 1992 */

say "Crude Particle System Program: Precipitation"

address "LightWaveARexx.port"
check = addlib("rexxmathlib.library",0,-30,0)

"CurrentObject 0"
frames=180
frames=450
dt=1/30
dt=1/120  /* Try temporal anti-aliasing */
framestep=1
startframe=1
time=dt*frames
cycles=time/6.283185
freq=20/cycles /* 20 flutters per animation loop */
zwind=1
xwind=2  /* Keep Integer for loop */
xmax=3
xmin=-3
xrng = xmax-xmin
ymax=4.5
ymin=0
yrng = ymax-ymin
zmax=6
zmin=0  /*  A plain of rain falls mainly at z=0 */
zrng = zmax-zmin
bounce=0 /* Factor like sqrt(elasticity) 0-1 */
bounceProb=0.2 /* Percent of particles which bounce */
fall_rate=2*yrng/time /*  Tune V for looping animation! (?) */
if bounce>0 then fall_rate=fall_rate*.4 /* Screws up loop... */
xrate=xrng/time /*  Tune winds for looping animation! (?) */
zrate=zrng/time /*  Tune winds for looping animation! (?) */
flake=.3
gusty=.5

flake=0
gusty=0
g=-9.8  /* m/s^2 */
say seed randu(time('S'))
say "Setting Initial Positions..."
do i = 0 to 999
    xp.i = xrng*randu() + xmin  /* Initial positions */
    yp.i = yrng*randu() + ymin
    zp.i = zrng*randu() + zmin
    xv.i = xwind*xrate
    yv.i = -fall_rate
    zv.i = zwind*zrate
    g.i=0 /* Deceleration for bounce  */
end i
say "Ready to Render!"
lwtime=0
do i = startframe to frames by framestep
  tm=time('R')
  say "Calculating frame " i " in "tm "Sec.s"
  do t=1 to framestep
      /*  gust should be same for all flakes at time t */
    gx=(1+gusty*0.4*(2*randu()-1)) /* Random-Walk style gusts in xv */
    gz=(1+gusty*0.4*(2*randu()-1)) /* different gusts in xv, zv */
    /* gy=(1+gusty*0.1*(2*randu()-1)) */ /* less gust in y */
    do j = 0 to 999
        "SetPoint "j" "xp.j" "yp.j" "zp.j   /* Position particles */
        if gusty>0 then do /* Screw up looping, change v by gusty*20% max step */
          xv.j=xv.j * gx
          zv.j=zv.j * gz
/*           yv.j=yv.j * gy */
        end
        if flake>0 then do
          gx=(1+flake*0.4*(2*randu()-1)) /* Random-Walk style gusts in xv */
          gz=(1+flake*0.4*(2*randu()-1)) /* different gusts in xv, zv */
          gy=(1+flake*0.1*(2*randu()-1)) /* less gust in y */
          xv.j=xv.j * gx  /* Try per-particle gusts as rough flake */
          zv.j=zv.j * gz
/*           yv.j=yv.j * gy */
/*        xv.j=xv.j + flake * sin(lwtime*freq+ph.j)
          zv.j=zv.j + flake * cos(lwtime*freq+ph.j)
          yv.j=yv.j + flake * sin(lwtime*freq+ph.j) */
        end
        yp.j = yp.j + yv.j*dt  /* Update positions */
        xp.j = xp.j + xv.j*dt
        zp.j = zp.j + zv.j*dt
        if bounce>0 & g.j then yv.j=yv.j+g*dt
        if zp.j<=zmin then zp.j=zp.j+zrng /* Recycle Rain */
        if zp.j>zmax then zp.j=zp.j-zrng
        if xp.j<=xmin then xp.j=xp.j+xrng
        if xp.j>xmax then xp.j=xp.j-xrng
        if yp.j<=ymin then  /* Bounce some percent or particles */
          if bounce>0 & randu()<=BounceProb then do
            yp.j=yp.j-yv.j*dt /* Reflect to prev yp */
            yv.j=-bounce*yv.j /* Reverse direction with energy loss */
            if g.j=1 then do /* Recycle on 2nd bounce, may now be unnecessary */
              yp.j=yp.j+yrng /* More natural to recycle when yv.j<thresh */
              yv.j = -fall_rate  /* Reset V */
              g.j=0  /* Turn off force, particles start at terminal velocity */
              end
            else g.j=1
            end
          else
             yp.j=yp.j+yrng
    end j
    lwtime=lwtime+dt
  end t
/* 	if i=startframe+2 then do
		say "First two frames rendered, hit <return> to continue"
		pull mydick
		end */
  if i>=startframe then do
   "FirstFrame "i
   "LastFrame "i
   "Render 1"
   end
end i

say "Thank you for using CPS!"

address

exit

